@@ -5,7 +5,9 @@ import boto3
 import docker
 import yaml
 import json
+import types
 import base64
+import builtins
 import argparse
 
 from textwrap import indent
@@ -17,10 +19,8 @@ from .utils import rgetattr, rsetattr, rhasattr, Colors
 class Deployment:
     def __init__(self, **kwargs):
         self.path = kwargs.get("path")
-        self.action = kwargs.get("action", "")
-        self.workflow = kwargs.get("workflow")
-        self.verbose = kwargs.get("verbose", True)
-        self.dry_run = kwargs.get("dry_run", False)
+        self.verbose = kwargs.get("verbose")
+        self.preview = kwargs.get("preview")
         self.config = yaml.safe_load(kwargs.get("config"))
 
         if not self.config:
@@ -33,6 +33,8 @@ class Deployment:
             "os": os,
             "re": re,
             "base64": base64,
+            "builtins": builtins,
+            "docker": docker.client.from_env(),
             "s3": boto3.client("s3"),
             "ecr": boto3.client("ecr"),
             "sts": boto3.client("sts"),
@@ -40,7 +42,7 @@ class Deployment:
             "iam": boto3.client("iam"),
             "events": boto3.client("events"),
             "lambda": boto3.client("lambda"),
-            "docker": docker.client.from_env(),
+            "organizations": boto3.client("organizations"),
         }
 
     def get_fully_qualified_node_names(self):
@@ -65,7 +67,7 @@ class Deployment:
         seen.add(node_name)
 
         for parent_node_name in node.get("depends_on"):
-            self.visit_parents(parent_node_name. seen, sorted_nodes)
+            self.visit_parents(parent_node_name, seen, sorted_nodes)
 
         sorted_nodes.append(node)
 
@@ -116,53 +118,69 @@ class Deployment:
             print(f"{Colors.OKCYAN}{node_name}{Colors.ENDC}")
 
         try:
+            args = self.lookup(node_args)
             action = rgetattr(node_client, action_name)
-            value = action(**self.lookup(node_args))
+
+            if isinstance(args, list):
+                value = action(*args)
+            elif isinstance(args, dict):
+                value = action(**args)
+
+            if isinstance(value, types.GeneratorType):
+                value = list(value)  # auto unpack generators
+
             if self.verbose:
-                print(indent(json.dumps(value, indent=2,
-                                        default=str), '+ ', lambda _: True))
+                self.print_status(value, Colors.OKGREEN)
 
             rsetattr(
                 context=self.ir,
                 accessor=f"{resource_name}.{action_name}",
-                value=value,
-            )
-        except Exception as e:
-            if self.verbose:
-                print(indent(json.dumps(e, indent=2, default=str),
-                             '- ', lambda _: True))
+                value=value)
 
-    def summarize(self, nodes):
+        except Exception as e:
+            self.print_status(e, Colors.FAIL)
+
+    def summarize(self, nodes, plan):
         print(f"{Colors.UNDERLINE}Executing nodes for: \
-               {self.workflow if self.workflow else self.action}{Colors.ENDC}")
+              {plan}{Colors.ENDC}")
 
         for node in nodes:
             print(f"{Colors.BOLD}{rgetattr(node, 'name')}{Colors.ENDC}")
 
-        print('------------------------------------------')
+        print()
 
-    def deploy(self):
+    def print_status(self, value, status):
+        indicator = "+" if status == Colors.OKGREEN else "-"
+        print(indent(json.dumps(value, indent=2,
+                                default=str),
+                     f"{status}{indicator} {Colors.ENDC}",
+                     lambda _: True))
+
+    def deploy(self, workflow):
         workflow_nodes = []
         for node in self.get_topologically_sorted_nodes():
-            if self.workflow in rgetattr(node, "workflow"):
+            if workflow in rgetattr(node, "workflow"):
                 workflow_nodes.append(node)
 
         if self.verbose:
-            self.summarize(workflow_nodes)
+            self.summarize(workflow_nodes, workflow)
 
-        if self.dry_run:
+        if self.preview:
             return
 
         for node in workflow_nodes:
             self.execute(node)
 
-    def one_off(self, node_name):
+    def one_off(self, node_name, include_deps):
         dependencies = self.visit_parents(node_name)
 
-        if self.verbose:
-            self.summarize(dependencies)
+        if not include_deps:
+            dependencies = [dependencies.pop()]
 
-        if self.dry_run:
+        if self.verbose:
+            self.summarize(dependencies, node_name)
+
+        if self.preview:
             return
 
         for node in dependencies:
@@ -172,23 +190,32 @@ class Deployment:
 def main():
     parser = argparse.ArgumentParser(description='Create an awyes deployment')
 
-    parser.add_argument('--dry_run', action=argparse.BooleanOptionalAction)
-    parser.add_argument('--workflow', type=str, required=False,
+    parser.add_argument(
+        '--preview', action=argparse.BooleanOptionalAction, default=False,
+        help="Whether or not to execute the plan")
+    parser.add_argument(
+        '--verbose', action=argparse.BooleanOptionalAction, default=True,
+        help="Enable logging")
+    parser.add_argument(
+        '--include-deps', action=argparse.BooleanOptionalAction, default=False,
+        help="When specifying an action, whether to include dependent actions")
+
+    parser.add_argument('--workflow', type=str, required=False, default="",
                         help='The awyes workflow type')
     parser.add_argument('--path', type=str, required=False,
                         default="awyes.yml", help='Path to awyes config')
     parser.add_argument('--config', type=str, required=False, default="",
                         help='Raw config to use in place of path')
-    parser.add_argument('--action', type=str, required=False,
+    parser.add_argument('--action', type=str, required=False, default="",
                         help="A specific action to run")
 
     args = parser.parse_args()
     deployment = Deployment(**vars(args))
 
     if args.action:
-        deployment.one_off(args.action)
+        deployment.one_off(args.action, args.include_deps)
     elif args.workflow:
-        deployment.deploy()
+        deployment.deploy(args.workflow)
     else:
         raise "Please pass either a workflow tag or an action"
 
